@@ -1,6 +1,6 @@
 ##  RQuantLib -- R interface to the QuantLib libraries
 ##
-##  Copyright (C) 2002 - 2014  Dirk Eddelbuettel <edd@debian.org>
+##  Copyright (C) 2002 - 2016  Dirk Eddelbuettel <edd@debian.org>
 ##
 ##  This file is part of RQuantLib.
 ##
@@ -18,15 +18,18 @@
 ##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.
 
 EuropeanOption <- function(type, underlying, strike, dividendYield,
-                           riskFreeRate, maturity, volatility) {
+                           riskFreeRate, maturity, volatility,
+                           discreteDividends = NULL, discreteDividendsTimeUntil = NULL) {
     UseMethod("EuropeanOption")
 }
 
 EuropeanOption.default <- function(type, underlying, strike, dividendYield,
-                                   riskFreeRate, maturity, volatility) {
+                                   riskFreeRate, maturity, volatility,
+                                   discreteDividends = NULL, discreteDividendsTimeUntil = NULL) {
     type <- match.arg(type, c("call", "put"))
     val <- europeanOptionEngine(type, underlying, strike, dividendYield,
-                                riskFreeRate, maturity, volatility)
+                                riskFreeRate, maturity, volatility,
+                                discreteDividends, discreteDividendsTimeUntil)
     class(val) <- c("EuropeanOption", "Option")
     val
 }
@@ -34,19 +37,22 @@ EuropeanOption.default <- function(type, underlying, strike, dividendYield,
 AmericanOption <- function(type, underlying, strike, dividendYield,
                            riskFreeRate, maturity, volatility,
                            timeSteps=150, gridPoints=149,
-                           engine="BaroneAdesiWhaley") {
+                           engine="BaroneAdesiWhaley",
+                           discreteDividends = NULL, discreteDividendsTimeUntil = NULL) {
     UseMethod("AmericanOption")
 }
 
 AmericanOption.default <- function(type, underlying, strike, dividendYield,
                                    riskFreeRate, maturity, volatility,
                                    timeSteps=150, gridPoints=149,
-                                   engine="BaroneAdesiWhaley") {
+                                   engine="BaroneAdesiWhaley",
+                                   discreteDividends = NULL, discreteDividendsTimeUntil = NULL) {
     type <- match.arg(type, c("call", "put"))
     engine <- match.arg(engine, c("BaroneAdesiWhaley", "CrankNicolson"))
     val <- americanOptionEngine(type, underlying, strike, dividendYield,
                                 riskFreeRate, maturity, volatility,
-                                timeSteps, gridPoints, engine)
+                                timeSteps, gridPoints, engine,
+                                discreteDividends, discreteDividendsTimeUntil)
     class(val) <- c("AmericanOption","Option")
     val
 }
@@ -65,7 +71,7 @@ BinaryOption.default <- function(binType, type, excType, underlying, strike, div
     excType <- match.arg(excType, c("american", "european"))
     val <- binaryOptionEngine(binType, type, excType, underlying,
                               strike, dividendYield, riskFreeRate,
-                              maturity, volatility, cashPayoff) 
+                              maturity, volatility, cashPayoff)
     class(val) <- c("BinaryOption", "Option")
     val
 }
